package syndicus.domain.util;

import java.io.Serializable;
import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.text.NumberFormat;

/**
 * 
 * @author debeuckelaer.d
 * 
 */
public class Euro extends Number implements Serializable, Comparable<Number> {
	private static final MathContext mathContext = new MathContext(32,
			RoundingMode.HALF_UP);

	private final String value;

	public Euro(double value) {
		this(new BigDecimal(value));
	}

	public Euro(BigDecimal bigDecimal) {
		this.value = bigDecimal.setScale(mathContext.getPrecision(),
				mathContext.getRoundingMode()).toString();
	}

	public Euro() {
		this(0);
	}

	public static Euro newInstance() {
		return new Euro();
	}

	public Euro(String xml) {
		throw new UnsupportedOperationException("TODO");
	}

	public Euro add(Euro euro) {
		return add(new BigDecimal(euro.value));
	}

	public Euro add(double value) {
		return add(new BigDecimal(value, mathContext));
	}

	public Euro add(BigDecimal bigDecimal) {
		return new Euro(new BigDecimal(value).add(bigDecimal));
	}

	public Euro subtract(Euro euro) {
		return subtract(new BigDecimal(euro.value));
	}

	public Euro subtract(double value) {
		return subtract(new BigDecimal(value, mathContext));
	}

	public Euro subtract(BigDecimal bigDecimal) {
		return new Euro(new BigDecimal(value).subtract(bigDecimal));
	}

	public Euro multiply(Euro euro) {
		return multiply(new BigDecimal(euro.value));
	}

	public Euro multiply(double value) {
		return multiply(new BigDecimal(value, mathContext));
	}

	public Euro multiply(BigDecimal decimal) {
		return new Euro(new BigDecimal(value).multiply(decimal));
	}

	public Euro divide(Euro euro) {
		return divide(new BigDecimal(euro.value));
	}

	public Euro divide(double value) {
		return divide(new BigDecimal(value, mathContext));
	}

	public Euro divide(BigDecimal decimal) {
		return new Euro(new BigDecimal(value).divide(decimal));
	}

	public double getValue() {
		return getValue(mathContext);
	}

	public double getPreciceValue() {
		return new BigDecimal(value, mathContext).doubleValue();
	}

	public double getValue(MathContext context) {
		return (((int)(100d *  (new BigDecimal(value, context).doubleValue() + getRounding()))) / 100d);
	}
	private double getRounding(){
		if(isPositive()){
			return .005;
		}else{
			return -(.005);
		}
	}

	public String toString() {
		return NumberFormat.getCurrencyInstance().format(getValue());
	}

	public boolean isNul() {
		return getValue() == 0.0;
	}
	public boolean isPositive() {
		return new BigDecimal(value).doubleValue() > 0.0;
	}

	public boolean greaterThan(double value) {
		return getValue() > value;
	}

	public Euro invert() {
		return new Euro(-new BigDecimal(value, mathContext).doubleValue());
	}

	@Override
	public double doubleValue() {
		return getValue();
	}

	@Override
	public float floatValue() {
		return new Float(getValue()).floatValue();
	}

	@Override
	public int intValue() {
		return new Double(getValue()).intValue();
	}

	@Override
	public long longValue() {
		return new Double(getValue()).longValue();
	}

	public int compareTo(Number o) {
		return new Double(getValue()).compareTo(new Double(o.doubleValue()));
	}

	@Override
	public boolean equals(Object obj) {
		return this.hashCode() == (obj != null ? obj.hashCode() : 0);
	}

	@Override
	public int hashCode() {
		return value.hashCode();
	}

}
