package euro.test;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.envers.AuditReader;
import org.hibernate.envers.AuditReaderFactory;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

@Transactional
@Repository(value = "dao")
public class GenericDAOImpl<T extends AbstractBaseEntity, PK extends Serializable>
		implements GenericDAO<T, PK> {
	/** type */
	protected Class<T> type;

	/** the logger */
	private static final Log LOG = LogFactory.getLog(GenericDAOImpl.class);

	@PersistenceContext
	protected EntityManager entityManager;

	public GenericDAOImpl() {
		this.type = (Class<T>) AbstractBaseEntity.class;
	}

	/**
	 * Minimal constructor
	 * 
	 * @param t
	 *            type POJO hibernate
	 */
	public GenericDAOImpl(Class<T> t) {
		this.type = t;
	}

	/**
	 * read data
	 * 
	 * @param id
	 *            data id
	 * @return data
	 */
	public T read(PK id) {
		if (id == null) {
			throw new IllegalArgumentException("Id cannot be null or empty");
		}
		return entityManager.find(type, id);

	}

	public void persist(T transientInstance) {
		if (LOG.isDebugEnabled()) {
			LOG.debug("Persisting instance of "
					+ transientInstance.getClass().getSimpleName());
		}
		entityManager.persist(transientInstance);

	}

	/**
	 * 
	 * attach clean
	 * 
	 * @param instance
	 *            data to attach
	 * @see sido.common.dao.GenericDAO#refresh(T)
	 */
	public void refresh(T instance) {
		if (LOG.isDebugEnabled()) {
			LOG.debug("refreshing instance of "
					+ instance.getClass().getSimpleName());
		}
		entityManager.refresh(instance);
	}

	/**
	 * delete
	 * 
	 * @param persistentInstance
	 *            data to delete
	 * @see sido.common.dao.GenericDAO#delete(T)
	 */
	public void delete(T persistentInstance) {
		if (LOG.isDebugEnabled()) {
			LOG.debug("deleting instance of "
					+ persistentInstance.getClass().getSimpleName());
		}
		entityManager.remove(persistentInstance);
	}

	/**
	 * merge
	 * 
	 * @param detachedInstance
	 *            data to merge
	 * @return the merged data
	 * @see sido.common.dao.GenericDAO#merge(T)
	 */
	public T merge(T detachedInstance) {
		if (LOG.isDebugEnabled()) {
			LOG.debug("merging instance of "
					+ detachedInstance.getClass().getSimpleName());
		}
		T merge = entityManager.merge(detachedInstance);
		return merge;

	}

	/**
	 * @return the entityManager
	 */
	public EntityManager getEntityManager() {
		return entityManager;
	}

	@SuppressWarnings("unchecked")
	public Collection<T> findAll() {
		List resultList = entityManager.createQuery("from " + type.getSimpleName())
				.getResultList();
		//Thread.dumpStack();
		return resultList;
	}
	public Criteria getCriteria(){
		Session session = (Session) getEntityManager().getDelegate();
		Criteria criteria = session.createCriteria(type);
		criteria.setCacheable(true);
		return criteria;
	}

	public void persist(Collection<T> coll) {
		for (T t : coll) {
			persist(t);
		}
		
	}

	@SuppressWarnings("unchecked")
	public T getLatestVersion(T t) {
		return (T)read((PK) t.getId());
	}

	private AuditReader getAuditReader() {
		return AuditReaderFactory.get(getEntityManager());
	}

	@SuppressWarnings("unchecked")
	public List<T> getAudits(Class<T> t) {
		return getAuditReader().createQuery().forRevisionsOfEntity(t, true,
				false).getResultList();
	}
}