package euro.test;

import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.BeanFactoryUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

public class ServiceLocator implements ApplicationContextAware,
InitializingBean {

	private static final String SERVICE_LOCATOR_WAS_NOT_PROPERLY_INITIALIZED = "ServiceLocator was not properly initialized, initialize SpringContext";

	private ApplicationContext context;

	private Log logger = LogFactory.getLog(getClass());

	private static final ServiceLocator instance = new ServiceLocator();

	public void setApplicationContext(ApplicationContext applicationContext)
	throws BeansException {
		context = applicationContext;
	}

	public void afterPropertiesSet() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("initialized");
		}
	}

	protected static ServiceLocator getInstance() {
		return instance;
	}

	/**
	 * Return a single bean of the given type or subtypes
	 * 
	 * @param serviceInterface
	 *            type of bean to match
	 * @return the matching bean instance
	 */
	@SuppressWarnings("unchecked")
	public <T> T locateService(Class<T> serviceInterface) {
		T result = null;
		if (this.getApplicationContext() == null)
			throw new IllegalStateException(
					SERVICE_LOCATOR_WAS_NOT_PROPERLY_INITIALIZED);
		if (logger.isDebugEnabled()) {
			logger.debug("locate : " + serviceInterface);
		}
		try {
			result = (T) BeanFactoryUtils.beanOfType(this
					.getApplicationContext(), serviceInterface);
		} catch (NoSuchBeanDefinitionException e) {
			return locate2(serviceInterface);
		}
		return result;
	}

	@SuppressWarnings("unchecked")
	<T> T locate2(Class<T> serviceInterface) {
		if (this.getApplicationContext() == null)
			throw new IllegalStateException(
					SERVICE_LOCATOR_WAS_NOT_PROPERLY_INITIALIZED);
		if (logger.isDebugEnabled()) {
			logger.debug("locate : " + serviceInterface);
		}
		Map beansOfTypeIncludingAncestors = BeanFactoryUtils
		.beansOfTypeIncludingAncestors(this.getApplicationContext(),
				serviceInterface);
		for (Object entry : beansOfTypeIncludingAncestors.entrySet()) {
			T value = (T) ((Map.Entry) entry).getValue();
			if (serviceInterface.isInstance(value)) {
				return value;
			}
			String valueClassName = value.getClass().getName();
			String className = serviceInterface.getName();
			if (valueClassName.equals(className)) {
				return value;
			}
		}
		return (T) BeanFactoryUtils.beanOfTypeIncludingAncestors(this
				.getApplicationContext(), serviceInterface);
	}

	@SuppressWarnings("unchecked")
	public <T> T locateByClassName(Class<T> serviceInterface)
	throws NoSuchBeanDefinitionException {
		T result = null;
		if (this.getApplicationContext() == null)
			throw new IllegalStateException(
					SERVICE_LOCATOR_WAS_NOT_PROPERLY_INITIALIZED);
		if (logger.isDebugEnabled()) {
			logger.debug("locate : " + serviceInterface);
		}
		String[] matchingBeanNames = BeanFactoryUtils
		.beanNamesForTypeIncludingAncestors(this
				.getApplicationContext(), serviceInterface);
		for (String beanName : matchingBeanNames) {
			Object bean = getApplicationContext().getBean(beanName);
			if (bean.getClass().equals(serviceInterface)) {
				result = (T) bean;
			}
		}
		if (result == null) {
			throw new NoSuchBeanDefinitionException(serviceInterface.getClass()
					.getName());
		}
		return result;
	}

	/**
	 * Return a single bean of the given type or subtypes
	 * 
	 * @param serviceInterface
	 *            type of bean to match
	 * @return the matching bean instance
	 */
	public static <T> T getService(Class<T> serviceInterface) {
		return getInstance().locateService(serviceInterface);
	}

	public static Object getBean(String name) {
		return getInstance().getBeanWithName(name);
	}

	private Object getBeanWithName(String name) {
		return getApplicationContext().getBean(name);
	}

	private ApplicationContext getApplicationContext() {
		return context;
	}

	public static ApplicationContext getContext() {
		return getInstance().getApplicationContext();
	}

}
